@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 # Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
@@ -9,10 +8,12 @@
 Test utilities.
 """
 
+from __future__ import print_function
 import atexit
 import contextlib
 import errno
 import functools
+import ipaddress  # python >= 3.3 / requires "pip install ipaddress"
 import os
 import re
 import shutil
@@ -21,16 +22,14 @@ import stat
 import subprocess
 import sys
 import tempfile
+import textwrap
 import threading
 import time
 import warnings
 from socket import AF_INET
 from socket import SOCK_DGRAM
 from socket import SOCK_STREAM
-try:
-    import ipaddress  # python >= 3.3
-except ImportError:
-    ipaddress = None
+
 try:
     from unittest import mock  # py3
 except ImportError:
@@ -38,7 +37,6 @@ except ImportError:
 
 import psutil
 from psutil import LINUX
-from psutil import OSX
 from psutil import POSIX
 from psutil import WINDOWS
 from psutil._compat import PY3
@@ -63,10 +61,36 @@ else:
     import imp as importlib
 
 
+__all__ = [
+    # constants
+    'APPVEYOR', 'DEVNULL', 'GLOBAL_TIMEOUT', 'MEMORY_TOLERANCE', 'NO_RETRIES',
+    'PYPY', 'PYTHON', 'RLIMIT_SUPPORT', 'ROOT_DIR', 'SCRIPTS_DIR',
+    'TESTFILE_PREFIX', 'TESTFN', 'TESTFN_UNICODE', 'TOX', 'TRAVIS',
+    'VALID_PROC_STATUSES', 'VERBOSITY',
+    # classes
+    'ThreadTask'
+    # test utils
+    'check_connection_ntuple', 'check_net_address', 'unittest', 'cleanup',
+    'skip_on_access_denied', 'skip_on_not_implemented', 'retry_before_failing',
+    'run_test_module_by_name',
+    # fs utils
+    'chdir', 'safe_rmpath', 'create_exe',
+    # subprocesses
+    'pyrun', 'reap_children', 'get_test_subprocess',
+    # os
+    'get_winver', 'get_kernel_version',
+    # sync primitives
+    'call_until', 'wait_for_pid', 'wait_for_file',
+    # others
+    'warn', 'decode_path', 'encode_path',
+]
+
+
 # ===================================================================
-# --- Constants
+# --- constants
 # ===================================================================
 
+
 # conf for retry_before_failing() decorator
 NO_RETRIES = 10
 # bytes tolerance for OS memory related tests
@@ -78,19 +102,22 @@ AF_INET6 = getattr(socket, "AF_INET6")
 AF_UNIX = getattr(socket, "AF_UNIX", None)
 PYTHON = os.path.realpath(sys.executable)
 DEVNULL = open(os.devnull, 'r+')
-TESTFN = os.path.join(os.getcwd(), "$testfile")
-TESTFN_UNICODE = TESTFN + "ƒőő"
-TESTFILE_PREFIX = 'psutil-test-suite-'
-TOX = os.getenv('TOX') or '' in ('1', 'true')
-PYPY = '__pypy__' in sys.builtin_module_names
+
+TESTFILE_PREFIX = '$psutil'
+TESTFN = os.path.join(os.path.realpath(os.getcwd()), TESTFILE_PREFIX)
+_TESTFN = TESTFN + '-internal'
+TESTFN_UNICODE = TESTFN + "-ƒőő"
 if not PY3:
     try:
-        TESTFN_UNICODE = unicode(TESTFN_UNICODE, sys.getfilesystemencoding())
+        TESTFN_UNICODE = unicode(TESTFN, sys.getfilesystemencoding())
     except UnicodeDecodeError:
-        TESTFN_UNICODE = TESTFN + "???"
+        TESTFN_UNICODE = TESTFN + "-???"
+
+TOX = os.getenv('TOX') or '' in ('1', 'true')
+PYPY = '__pypy__' in sys.builtin_module_names
 
 ROOT_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__),
-                           '..', '..'))
+                                        '..', '..'))
 SCRIPTS_DIR = 'scripts'
 
 WIN_VISTA = (6, 0, 0) if WINDOWS else None
@@ -102,17 +129,21 @@ TRAVIS = bool(os.environ.get('TRAVIS'))
 # (http://www.appveyor.com/)
 APPVEYOR = bool(os.environ.get('APPVEYOR'))
 
-if TRAVIS or 'tox' in sys.argv[0]:
-    import ipaddress
 if TRAVIS or APPVEYOR:
     GLOBAL_TIMEOUT = GLOBAL_TIMEOUT * 4
 VERBOSITY = 1 if os.getenv('SILENT') or TOX else 2
 
+# assertRaisesRegexp renamed to assertRaisesRegex in 3.3; add support
+# for the new name
+if not hasattr(unittest.TestCase, 'assertRaisesRegex'):
+    unittest.TestCase.assertRaisesRegex = unittest.TestCase.assertRaisesRegexp
+
 
 # ===================================================================
-# --- Classes
+# --- classes
 # ===================================================================
 
+
 class ThreadTask(threading.Thread):
     """A thread object used for running process thread tests."""
 
@@ -151,60 +182,34 @@ class ThreadTask(threading.Thread):
 
 
 # ===================================================================
-# --- Utility functions
+# --- subprocesses
 # ===================================================================
 
-def cleanup():
-    reap_children(search_all=True)
-    safe_remove(TESTFN)
-    try:
-        safe_rmdir(TESTFN_UNICODE)
-    except UnicodeEncodeError:
-        pass
-    for path in _testfiles:
-        safe_remove(path)
-
-atexit.register(cleanup)
-atexit.register(lambda: DEVNULL.close())
-
 
 _subprocesses_started = set()
 
 
-def get_test_subprocess(cmd=None, wait=False, **kwds):
+def get_test_subprocess(cmd=None, **kwds):
     """Return a subprocess.Popen object to use in tests.
     By default stdout and stderr are redirected to /dev/null and the
     python interpreter is used as test process.
-    If 'wait' is True attemps to make sure the process is in a
-    reasonably initialized state.
+    It also attemps to make sure the process is in a reasonably
+    initialized state.
     """
-    if cmd is None:
-        pyline = ""
-        if wait:
-            pyline += "open(r'%s', 'w'); " % TESTFN
-        # A process living for 30 secs. We sleep N times (as opposed to
-        # once) in order to be nicer towards Windows which doesn't handle
-        # interrupt signals properly.
-        pyline += "import time; [time.sleep(0.01) for x in range(3000)];"
-        cmd_ = [PYTHON, "-c", pyline]
-    else:
-        cmd_ = cmd
     kwds.setdefault("stdin", DEVNULL)
     kwds.setdefault("stdout", DEVNULL)
-    kwds.setdefault("stderr", DEVNULL)
-    sproc = subprocess.Popen(cmd_, **kwds)
-    if wait:
-        if cmd is None:
-            stop_at = time.time() + 3
-            while stop_at > time.time():
-                if os.path.exists(TESTFN):
-                    break
-                time.sleep(0.001)
-            else:
-                warn("couldn't make sure test file was actually created")
-        else:
-            wait_for_pid(sproc.pid)
-    _subprocesses_started.add(psutil.Process(sproc.pid))
+    if cmd is None:
+        assert not os.path.exists(_TESTFN)
+        pyline = "from time import sleep;"
+        pyline += "open(r'%s', 'w').close();" % _TESTFN
+        pyline += "sleep(60)"
+        cmd = [PYTHON, "-c", pyline]
+        sproc = subprocess.Popen(cmd, **kwds)
+        wait_for_file(_TESTFN, delete_file=True, empty=True)
+    else:
+        sproc = subprocess.Popen(cmd, **kwds)
+        wait_for_pid(sproc.pid)
+    _subprocesses_started.add(sproc)
     return sproc
 
 
@@ -212,7 +217,7 @@ _testfiles = []
 
 
 def pyrun(src):
-    """Run python code 'src' in a separate interpreter.
+    """Run python 'src' code in a separate interpreter.
     Return interpreter subprocess.
     """
     if PY3:
@@ -228,11 +233,6 @@ def pyrun(src):
         return subp
 
 
-def warn(msg):
-    """Raise a warning msg."""
-    warnings.warn(msg, UserWarning)
-
-
 def sh(cmdline, stdout=subprocess.PIPE, stderr=subprocess.PIPE):
     """run cmd in a subprocess and return its output.
     raises RuntimeError on error.
@@ -252,7 +252,77 @@ def sh(cmdline, stdout=subprocess.PIPE, stderr=subprocess.PIPE):
     return stdout.strip()
 
 
-if POSIX:
+def reap_children(recursive=False):
+    """Terminate and wait() any subprocess started by this test suite
+    and ensure that no zombies stick around to hog resources and
+    create problems  when looking for refleaks.
+
+    If resursive is True it also tries to terminate and wait()
+    all grandchildren started by this process.
+    """
+    # Get the children here, before terminating the children sub
+    # processes as we don't want to lose the intermediate reference
+    # in case of grandchildren.
+    if recursive:
+        children = psutil.Process().children(recursive=True)
+    else:
+        children = []
+
+    # Terminate subprocess.Popen instances "cleanly" by closing their
+    # fds and wiat()ing for them in order to avoid zombies.
+    subprocs = _subprocesses_started.copy()
+    _subprocesses_started.clear()
+    for subp in subprocs:
+        try:
+            subp.terminate()
+        except OSError as err:
+            if err.errno != errno.ESRCH:
+                raise
+        if subp.stdout:
+            subp.stdout.close()
+        if subp.stderr:
+            subp.stderr.close()
+        try:
+            # Flushing a BufferedWriter may raise an error.
+            if subp.stdin:
+                subp.stdin.close()
+        finally:
+            # Wait for the process to terminate, to avoid zombies.
+            try:
+                subp.wait()
+            except OSError as err:
+                if err.errno != errno.ECHILD:
+                    raise
+
+    # Terminates grandchildren.
+    if children:
+        for p in children:
+            try:
+                p.terminate()
+            except psutil.NoSuchProcess:
+                pass
+        gone, alive = psutil.wait_procs(children, timeout=GLOBAL_TIMEOUT)
+        for p in alive:
+            warn("couldn't terminate process %r; attempting kill()" % p)
+            try:
+                p.kill()
+            except psutil.NoSuchProcess:
+                pass
+        _, alive = psutil.wait_procs(alive, timeout=GLOBAL_TIMEOUT)
+        if alive:
+            for p in alive:
+                warn("process %r survived kill()" % p)
+
+
+# ===================================================================
+# --- OS
+# ===================================================================
+
+
+if not POSIX:
+    def get_kernel_version():
+        return ()
+else:
     def get_kernel_version():
         """Return a tuple such as (2, 6, 36)."""
         s = ""
@@ -273,9 +343,6 @@ if POSIX:
         if len(nums) >= 3:
             micro = int(nums[2])
         return (major, minor, micro)
-else:
-    def get_kernel_version():
-        return ()
 
 
 if LINUX:
@@ -284,65 +351,248 @@ else:
     RLIMIT_SUPPORT = False
 
 
-def wait_for_pid(pid, timeout=GLOBAL_TIMEOUT):
+if not WINDOWS:
+    def get_winver():
+        raise NotImplementedError("not a Windows OS")
+else:
+    def get_winver():
+        wv = sys.getwindowsversion()
+        if hasattr(wv, 'service_pack_major'):  # python >= 2.7
+            sp = wv.service_pack_major or 0
+        else:
+            r = re.search("\s\d$", wv[4])
+            if r:
+                sp = int(r.group(0))
+            else:
+                sp = 0
+        return (wv[0], wv[1], sp)
+
+
+# ===================================================================
+# --- sync primitives
+# ===================================================================
+
+
+class retry(object):
+    """A retry decorator."""
+
+    def __init__(self,
+                 exception=Exception,
+                 timeout=None,
+                 retries=None,
+                 interval=0.001,
+                 logfun=lambda s: print(s, file=sys.stderr),
+                 ):
+        if timeout and retries:
+            raise ValueError("timeout and retries args are mutually exclusive")
+        self.exception = exception
+        self.timeout = timeout
+        self.retries = retries
+        self.interval = interval
+        self.logfun = logfun
+
+    def __iter__(self):
+        if self.timeout:
+            stop_at = time.time() + self.timeout
+            while time.time() < stop_at:
+                yield
+        elif self.retries:
+            for _ in range(self.retries):
+                yield
+        else:
+            while True:
+                yield
+
+    def sleep(self):
+        if self.interval is not None:
+            time.sleep(self.interval)
+
+    def __call__(self, fun):
+        @functools.wraps(fun)
+        def wrapper(*args, **kwargs):
+            exc = None
+            for _ in self:
+                try:
+                    return fun(*args, **kwargs)
+                except self.exception as _:
+                    exc = _
+                    if self.logfun is not None:
+                        self.logfun(exc)
+                    self.sleep()
+            else:
+                if PY3:
+                    raise exc
+                else:
+                    raise
+
+        # This way the user of the decorated function can change config
+        # parameters.
+        wrapper.decorator = self
+        return wrapper
+
+
+@retry(exception=psutil.NoSuchProcess, logfun=None, timeout=GLOBAL_TIMEOUT,
+       interval=0.001)
+def wait_for_pid(pid):
     """Wait for pid to show up in the process list then return.
     Used in the test suite to give time the sub process to initialize.
     """
-    raise_at = time.time() + timeout
-    while True:
-        if pid in psutil.pids():
-            # give it one more iteration to allow full initialization
-            time.sleep(0.01)
-            return
-        time.sleep(0.0001)
-        if time.time() >= raise_at:
-            raise RuntimeError("Timed out")
+    psutil.Process(pid)
+    if WINDOWS:
+        # give it some more time to allow better initialization
+        time.sleep(0.01)
 
 
-def wait_for_file(fname, timeout=GLOBAL_TIMEOUT, delete_file=True):
-    """Wait for a file to be written on disk."""
-    stop_at = time.time() + 3
-    while time.time() < stop_at:
-        try:
-            with open(fname, "r") as f:
-                data = f.read()
-            if not data:
-                continue
-            if delete_file:
-                os.remove(fname)
-            return data
-        except IOError:
-            time.sleep(0.001)
-    raise RuntimeError("timed out (couldn't read file)")
+@retry(exception=(EnvironmentError, AssertionError), logfun=None,
+       timeout=GLOBAL_TIMEOUT, interval=0.001)
+def wait_for_file(fname, delete_file=True, empty=False):
+    """Wait for a file to be written on disk with some content."""
+    with open(fname, "rb") as f:
+        data = f.read()
+    if not empty:
+        assert data
+    if delete_file:
+        os.remove(fname)
+    return data
 
 
-def reap_children(search_all=False):
-    """Kill any subprocess started by this test suite and ensure that
-    no zombies stick around to hog resources and create problems when
-    looking for refleaks.
+@retry(exception=AssertionError, logfun=None, timeout=GLOBAL_TIMEOUT,
+       interval=0.001)
+def call_until(fun, expr):
+    """Keep calling function for timeout secs and exit if eval()
+    expression is True.
     """
-    global _subprocesses_started
-    procs = _subprocesses_started.copy()
-    if search_all:
-        this_process = psutil.Process()
-        for p in this_process.children(recursive=True):
-            procs.add(p)
-    for p in procs:
+    ret = fun()
+    assert eval(expr)
+    return ret
+
+
+# ===================================================================
+# --- fs
+# ===================================================================
+
+
+def safe_rmpath(path):
+    "Convenience function for removing temporary test files or dirs"
+    try:
+        st = os.stat(path)
+        if stat.S_ISDIR(st.st_mode):
+            os.rmdir(path)
+        else:
+            os.remove(path)
+    except OSError as err:
+        if err.errno != errno.ENOENT:
+            raise
+
+
+@contextlib.contextmanager
+def chdir(dirname):
+    "Context manager which temporarily changes the current directory."
+    curdir = os.getcwd()
+    try:
+        os.chdir(dirname)
+        yield
+    finally:
+        os.chdir(curdir)
+
+
+def create_exe(outpath, c_code=None):
+    assert not os.path.exists(outpath), outpath
+    if which("gcc"):
+        if c_code is None:
+            c_code = textwrap.dedent(
+                """
+                #include <unistd.h>
+                int main() {
+                    pause();
+                    return 1;
+                }
+                """)
+        with tempfile.NamedTemporaryFile(
+                suffix='.c', delete=False, mode='wt') as f:
+            f.write(c_code)
         try:
-            p.terminate()
-        except psutil.NoSuchProcess:
-            pass
-    gone, alive = psutil.wait_procs(procs, timeout=GLOBAL_TIMEOUT)
-    for p in alive:
-        warn("couldn't terminate process %s" % p)
-        try:
-            p.kill()
-        except psutil.NoSuchProcess:
-            pass
-    _, alive = psutil.wait_procs(alive, timeout=GLOBAL_TIMEOUT)
-    if alive:
-        warn("couldn't not kill processes %s" % str(alive))
-    _subprocesses_started = set(alive)
+            subprocess.check_call(["gcc", f.name, "-o", outpath])
+        finally:
+            safe_rmpath(f.name)
+    else:
+        # fallback - use python's executable
+        shutil.copyfile(sys.executable, outpath)
+        if POSIX:
+            st = os.stat(outpath)
+            os.chmod(outpath, st.st_mode | stat.S_IEXEC)
+
+
+# ===================================================================
+# --- testing
+# ===================================================================
+
+
+class TestCase(unittest.TestCase):
+
+    def __str__(self):
+        return "%s.%s.%s" % (
+            self.__class__.__module__, self.__class__.__name__,
+            self._testMethodName)
+
+
+# Hack that overrides default unittest.TestCase in order to print
+# a full path representation of the single unit tests being run.
+unittest.TestCase = TestCase
+
+
+def retry_before_failing(retries=NO_RETRIES):
+    """Decorator which runs a test function and retries N times before
+    actually failing.
+    """
+    return retry(exception=AssertionError, timeout=None, retries=retries)
+
+
+def run_test_module_by_name(name):
+    # testmodules = [os.path.splitext(x)[0] for x in os.listdir(HERE)
+    #                if x.endswith('.py') and x.startswith('test_')]
+    name = os.path.splitext(os.path.basename(name))[0]
+    suite = unittest.TestSuite()
+    suite.addTest(unittest.defaultTestLoader.loadTestsFromName(name))
+    result = unittest.TextTestRunner(verbosity=VERBOSITY).run(suite)
+    success = result.wasSuccessful()
+    sys.exit(0 if success else 1)
+
+
+def skip_on_access_denied(only_if=None):
+    """Decorator to Ignore AccessDenied exceptions."""
+    def decorator(fun):
+        @functools.wraps(fun)
+        def wrapper(*args, **kwargs):
+            try:
+                return fun(*args, **kwargs)
+            except psutil.AccessDenied:
+                if only_if is not None:
+                    if not only_if:
+                        raise
+                msg = "%r was skipped because it raised AccessDenied" \
+                      % fun.__name__
+                raise unittest.SkipTest(msg)
+        return wrapper
+    return decorator
+
+
+def skip_on_not_implemented(only_if=None):
+    """Decorator to Ignore NotImplementedError exceptions."""
+    def decorator(fun):
+        @functools.wraps(fun)
+        def wrapper(*args, **kwargs):
+            try:
+                return fun(*args, **kwargs)
+            except NotImplementedError:
+                if only_if is not None:
+                    if not only_if:
+                        raise
+                msg = "%r was skipped because it raised NotImplementedError" \
+                      % fun.__name__
+                raise unittest.SkipTest(msg)
+        return wrapper
+    return decorator
 
 
 def check_net_address(addr, family):
@@ -356,16 +606,14 @@ def check_net_address(addr, family):
         assert len(octs) == 4, addr
         for num in octs:
             assert 0 <= num <= 255, addr
-        if ipaddress:
-            if not PY3:
-                addr = unicode(addr)
-            ipaddress.IPv4Address(addr)
+        if not PY3:
+            addr = unicode(addr)
+        ipaddress.IPv4Address(addr)
     elif family == AF_INET6:
         assert isinstance(addr, str), addr
-        if ipaddress:
-            if not PY3:
-                addr = unicode(addr)
-            ipaddress.IPv6Address(addr)
+        if not PY3:
+            addr = unicode(addr)
+        ipaddress.IPv6Address(addr)
     elif family == psutil.AF_LINK:
         assert re.match('([a-fA-F0-9]{2}[:|\-]?){6}', addr) is not None, addr
     else:
@@ -432,146 +680,29 @@ def check_connection_ntuple(conn):
                     assert dupsock.type == conn.type
 
 
-def safe_remove(file):
-    "Convenience function for removing temporary test files"
-    try:
-        os.remove(file)
-    except OSError as err:
-        if err.errno != errno.ENOENT:
-            # # file is being used by another process
-            # if WINDOWS and isinstance(err, WindowsError) and err.errno == 13:
-            #     return
-            raise
-
-
-def safe_rmdir(dir):
-    "Convenience function for removing temporary test directories"
-    try:
-        os.rmdir(dir)
-    except OSError as err:
-        if err.errno != errno.ENOENT:
-            raise
-
-
-@contextlib.contextmanager
-def chdir(dirname):
-    """Context manager which temporarily changes the current directory."""
-    curdir = os.getcwd()
-    try:
-        os.chdir(dirname)
-        yield
-    finally:
-        os.chdir(curdir)
-
-
-def call_until(fun, expr, timeout=GLOBAL_TIMEOUT):
-    """Keep calling function for timeout secs and exit if eval()
-    expression is True.
-    """
-    stop_at = time.time() + timeout
-    while time.time() < stop_at:
-        ret = fun()
-        if eval(expr):
-            return ret
-        time.sleep(0.001)
-    raise RuntimeError('timed out (ret=%r)' % ret)
-
-
-def retry_before_failing(ntimes=None):
-    """Decorator which runs a test function and retries N times before
-    actually failing.
-    """
-    def decorator(fun):
-        @functools.wraps(fun)
-        def wrapper(*args, **kwargs):
-            times = ntimes or NO_RETRIES
-            assert times, times
-            for x in range(times):
-                try:
-                    return fun(*args, **kwargs)
-                except AssertionError as _:
-                    err = _
-            raise err
-        return wrapper
-    return decorator
-
-
-def skip_on_access_denied(only_if=None):
-    """Decorator to Ignore AccessDenied exceptions."""
-    def decorator(fun):
-        @functools.wraps(fun)
-        def wrapper(*args, **kwargs):
+def cleanup():
+    for name in os.listdir('.'):
+        if name.startswith(TESTFILE_PREFIX):
             try:
-                return fun(*args, **kwargs)
-            except psutil.AccessDenied:
-                if only_if is not None:
-                    if not only_if:
-                        raise
-                msg = "%r was skipped because it raised AccessDenied" \
-                      % fun.__name__
-                raise unittest.SkipTest(msg)
-        return wrapper
-    return decorator
+                safe_rmpath(name)
+            except UnicodeEncodeError as exc:
+                warn(exc)
+    for path in _testfiles:
+        safe_rmpath(path)
 
 
-def skip_on_not_implemented(only_if=None):
-    """Decorator to Ignore NotImplementedError exceptions."""
-    def decorator(fun):
-        @functools.wraps(fun)
-        def wrapper(*args, **kwargs):
-            try:
-                return fun(*args, **kwargs)
-            except NotImplementedError:
-                if only_if is not None:
-                    if not only_if:
-                        raise
-                msg = "%r was skipped because it raised NotImplementedError" \
-                      % fun.__name__
-                raise unittest.SkipTest(msg)
-        return wrapper
-    return decorator
+atexit.register(cleanup)
+atexit.register(lambda: DEVNULL.close())
 
 
-def create_temp_executable_file(suffix, code="void main() { pause(); }"):
-    tmpdir = None
-    if TRAVIS and OSX:
-        tmpdir = "/private/tmp"
-    fd, path = tempfile.mkstemp(
-        prefix='psu', suffix=suffix, dir=tmpdir)
-    os.close(fd)
-
-    if which("gcc"):
-        fd, c_file = tempfile.mkstemp(
-            prefix='psu', suffix='.c', dir=tmpdir)
-        os.close(fd)
-        with open(c_file, "w") as f:
-            f.write(code)
-        subprocess.check_call(["gcc", c_file, "-o", path])
-        safe_remove(c_file)
-    else:
-        # fallback - use python's executable
-        shutil.copyfile(sys.executable, path)
-        if POSIX:
-            st = os.stat(path)
-            os.chmod(path, st.st_mode | stat.S_IEXEC)
-    return path
+# ===================================================================
+# --- others
+# ===================================================================
 
 
-if WINDOWS:
-    def get_winver():
-        wv = sys.getwindowsversion()
-        if hasattr(wv, 'service_pack_major'):  # python >= 2.7
-            sp = wv.service_pack_major or 0
-        else:
-            r = re.search("\s\d$", wv[4])
-            if r:
-                sp = int(r.group(0))
-            else:
-                sp = 0
-        return (wv[0], wv[1], sp)
-else:
-    def get_winver():
-        raise NotImplementedError("not a Windows OS")
+def warn(msg):
+    """Raise a warning msg."""
+    warnings.warn(msg, UserWarning)
 
 
 # In Python 3 paths are unicode objects by default.  Surrogate escapes
@@ -590,14 +721,3 @@ def decode_path(path):
                            errors="surrogateescape")
     else:
         return path
-
-
-def run_test_module_by_name(name):
-    # testmodules = [os.path.splitext(x)[0] for x in os.listdir(HERE)
-    #                if x.endswith('.py') and x.startswith('test_')]
-    name = os.path.splitext(os.path.basename(name))[0]
-    suite = unittest.TestSuite()
-    suite.addTest(unittest.defaultTestLoader.loadTestsFromName(name))
-    result = unittest.TextTestRunner(verbosity=VERBOSITY).run(suite)
-    success = result.wasSuccessful()
-    sys.exit(0 if success else 1)
